/* Copyright (c) 1996 by Stanford University.  All Rights Reserved */

/***************************************************************************
   NAME

   PURPOSE
     
   NOTES
     
   AUTHOR
     David Hsu  
   
     dyhsu@cs.stanford.edu
     http://robotics.stanford.edu/~dyhsu 
                  
   HISTORY

***************************************************************************/


#ifndef _PLANNER1_H
#define _PLANNER1_H

#include "basicPlanner.H"
#include "utilities.H"
#include "configArray.H"

/* Tray
 *   a set of config's that form a connected component in c-space
 *   contain routines to expand the known c-space
 *
 *   Color assignment of nodes
 *      white    :    a  node that has not been checked for expansion
 *      gray     :    a  node that has  been checked for expansion
 *      dead     :    a node that is stopped from further expansion
 *
 *      black    :    a  node hat has been checked for connection with
 *                    the other component                       
 */

const static int INIT_SIZE = 1024;	/* initial size of Tray */ 

class Tray {
    friend ostream& operator<<(ostream& os, Tray& t);

public:
    unsigned white;			/* mark the end of white nodes */
    unsigned gray;			/* mark the end of gray nodes  */
    unsigned black;			/* mark the end of black nodes */
    unsigned dead;			/* mark the end of dead nodes   */

    Tray() { white =gray =black =dead =0; qs.resize(INIT_SIZE);
	     r=NULL; obs=NULL; csp=NULL; box=NULL; }
    
    void bgn( Robot* robot, Obstacle* obstacle, 
	      Cspace* cspace, SampleBox* samplebox);
    PtrConfig& operator[](int i) {return qs[i];}
    Bool reset( Config& n );
    void expand();
    inline Bool sample(Config& parent);
    Bool getChild(Config& parent, Config& child);
    void dump(char* fname);

private:
    Robot* r;                            /* robot in workspace    */
    Obstacle* obs;                       /* obstacle in workspace */
    Cspace* csp;                         /*  c-space   */
    SampleBox* box;			 /* sample box */

    ConfigArray qs;
};


/* Planner1
 *    plan by expanding from the known c-space
 */

class Config;				/* forward decl */

class Planner1: public BasicPlanner {
public:
    void bgn(istream& rsrc, istream& osrc, istream &asrc); 
    void query(ostream& pfile);
  
protected:
    Tray icomp;
    Tray gcomp;
    SampleBox* box;			/* sample box */

    Bool connectComp(Config* &iConfig, Config* &gConfig);
};


#endif /* _PLANNER1_H */
