/* Copyright (c) 1996 by Stanford University.  All Rights Reserved */

/***************************************************************************
   NAME

   PURPOSE
     
   NOTES
     
   AUTHOR
     David Hsu  
   
     dyhsu@cs.stanford.edu
     http://robotics.stanford.edu/~dyhsu 
                  
   HISTORY

***************************************************************************/


#include <iostream.h>
#include <limits.h>
#include <unistd.h>
#include "stat.H"

/* reset
 *   reset all the statistics
 */

void Statistics::reset()
{
    totalRawSample = totalSample = totalFree = totalFreeCnt = 0UL;
    iWhite = iGray = iDead = 0UL;
    gWhite = gGray = gDead = 0UL;
    numColCheck = numConnection = numConnectPair = numLink = 0UL;
    numIter = 0;
}


ostream& operator<< (ostream& os, Statistics& s)
{
    os << "Statistics:\n";
    os << "\t Name of robot\t\t\t= " << s.robotName << "\n";
    os << "\t Name of environment\t\t= " << s.obsName << "\n\n";
    os << "\t total raw samples\t\t= " << s.totalRawSample << "\n";
    os << "\t total c-space samples\t\t= " << s.totalSample << "\n";
    os << "\t total free sample points\t= " << s.totalFree << "\n";
    os << "\t total connected free samples\t= " << s.totalFreeCnt << "\n";
    os << "\t fraction of free sample\t= " << 
           double(s.totalFree) / s.totalSample << "\n\n";
    os << "\t Collision checks\t\t= " << s.numColCheck << "\n";
    os << "\t number of conections attempted within a component\t= " 
        << s.numLink  << "\n";
    os << "\t number of connections attempted between components \t= "
        << s.numConnection << "\n";
    os << "\t number of calls to connectPair \t\t\t= "
        << s.numConnectPair << "\n\n";

    os << "\t number of white nodes connected to initial Config.\t= " 
         << ( s.iWhite - s.iGray) << "\n";
    os << "\t number of gray nodes connected to initial Config.\t= " 
         <<  (s.iGray - s.iDead ) << "\n";
    os << "\t number of dead nodes connected to initial Config.\t= " 
         <<  s.iDead << "\n\n";

    os << "\t number of white nodes connected to goal Config.\t= " 
         << ( s.gWhite - s.gGray) << "\n";
    os << "\t number of gray nodes connected to goal Config.\t\t= " 
         << ( s.gGray - s.gDead ) << "\n";
    os << "\t number of dead nodes connected to goal Config.\t\t= " 
         << s.gDead  << "\n\n";
    
    os << "Execution time:\n";
    float ticks = CLK_TCK;
    float utime = (s.qEnd.tms_utime - s.qStart.tms_utime) / ticks;
    float stime = (s.qEnd.tms_stime - s.qStart.tms_stime) / ticks;
    os << "\t User time\t= " << utime << endl;
    os << "\t System time\t= " << stime << endl;
    os << "\t Total time\t= " << utime + stime << "\n\n";

    return os;
}


