/* Copyright (c) 1996 by Stanford University.  All Rights Reserved */

/***************************************************************************
   NAME

   PURPOSE
     
   NOTES
     
   AUTHOR
     David Hsu  
   
     dyhsu@cs.stanford.edu
     http://robotics.stanford.edu/~dyhsu 
                  
   HISTORY

***************************************************************************/


/* Copyright (c) 1996 by Stanford University.  All Rights Reserved */

/***************************************************************************
   NAME
     utilites.h

  PURPOSE
     low level constants, data types and routines useful for all modules

   NOTES
     
   AUTHOR
     David Hsu  
   
     dyhsu@cs.stanford.edu
     http://robotics.stanford.edu/~dyhsu 
                  
   HISTORY

***************************************************************************/



#ifndef _UTILITIES_H
#define _UTILITIES_H

#include <iostream.h>
#include "random.H"
#include "dist.h"

/*----------------------   Constants   -----------------------------------*/

#define	SUCCESS		1
#define FAILURE		-1
#define FALSE 		0
#define TRUE		1
#define	X		0
#define Y		1
#define Z		2

extern Frame3 IFRAME[1];		/* represent identity tranformation */

extern RandomDouble randDbl;		/*  random double generator */
extern RandomInt    randInt;		/*  random int generator    */



/*----------------------   Types       -----------------------------------*/

typedef int 	Bool;

/* type for the coordinates of C-space
 */
typedef float 	Coord;
typedef Coord	Vec3[3];



/*----------------------- Functions    -----------------------------------*/
void error(char* msg, char* arg=NULL);
void stripComment(istream& src, ostream& target);

template <class T> 
inline T MAX(T A, T B) 
{
    return ( A > B) ? A : B; 
}

template <class T> 
inline T MIN(T A, T B) 
{
    return ( A < B) ? A : B; 
}

template <class T> 
inline T ABS(T A) 
{
    return ( A >= 0) ? A : -A; 
}

template <class T>
inline void swap(T& a, T& b)
{
    T tmp;
    tmp = a; a = b; b = tmp;
}


/* QuatMul
 *   quaternion multiplication r = q p
 *
 *   Comment: Don't do QualMul(q, p, q) !!! 
 */

inline void QuatMul(Coord* r, Coord* q, Coord* p)
{
    r[0] = q[0]*p[0] - q[1]*p[1] - q[2]*p[2] - q[3]*p[3];
    r[1] = q[0]*p[1] + q[1]*p[0] + q[2]*p[3] - q[3]*p[2];
    r[2] = q[0]*p[2] - q[1]*p[3] + q[2]*p[0] + q[3]*p[1];
    r[3] = q[0]*p[3] + q[1]*p[2] - q[2]*p[1] + q[3]*p[0];
}


/* QuatMulConj
 *   r = q conj(p)
 */

inline void QuatMulConj(Coord* r, Coord* q, Coord* p)
{
    r[0] =  q[0]*p[0] + q[1]*p[1] + q[2]*p[2] + q[3]*p[3];
    r[1] = -q[0]*p[1] + q[1]*p[0] - q[2]*p[3] + q[3]*p[2];
    r[2] = -q[0]*p[2] + q[1]*p[3] + q[2]*p[0] - q[3]*p[1];
    r[3] = -q[0]*p[3] - q[1]*p[2] + q[2]*p[1] + q[3]*p[0];
}


#endif  /* _UTILITIES_H */
